             .title          "NSF Loader Plugin"


;06/17/00 
;Written by KH
;Version 1.0
             
;protocol for reading:             
;
;send out size of PRG ROM (two bytes, LSB first) # of 256 byte pages
;indicate PRG ROM (01h = PRG)
;send out bytes for PRG
;send out size of CHR ROM (two bytes, LSB first) # of 256 byte pages
;indicate CHR ROM (02h = CHR)
;send out stop (00h)


             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

inbase:      .equ 01f0h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

temp1:       .equ 00e0h
temp1_lo:    .equ 00e0h
temp1_hi:    .equ 00e1h
temp2:       .equ 00e2h
temp2_lo:    .equ 00e2h
temp2_hi:    .equ 00e3h
temp3:       .equ 00e4h
temp3_lo:    .equ 00e4h
temp3_hi:    .equ 00e5h

             .org 0280h

load:        .block 2
init:        .block 2
play:        .block 2
len:         .block 3

banks:       .block 8
initsng:     .block 1
numsng:      .block 1


in_mode:     .equ 0304bh



             ;plugin header that describes what it does
             
             .org 0380h
             
             .db "NSF Loader plugin" 
             .db 0

             .fill 0400h-*,00h    ;all plugins must reside at 400h

             lda #0
             sta 04803h        ;set to input mode


             lda #01fh
             sta temp2         ;# banks

df_lp1:      ldx #010h
             lda #0
             sta temp1_lo
             lda #080h
             sta temp1_hi
             lda temp2
             sta 5ff8h
             ldy #0            ;init bank # and pointers
             tya

df_lp2:      sta (temp1),y
             iny
             bne df_lp2
             inc temp1_hi
             dex
             bne df_lp2        ;clear all NSF RAM
             dec temp2
             bpl df_lp1        ;all banks
             
             ldx #060h
             stx temp1_hi
             lda #0
             sta temp1_lo
             tay

cd_lk:       sta (temp1),y
             iny
             bne cd_lk
             inc temp1_hi
             bne cd_lk         ;clear RAM at 6000-7FFF
             ldy #0

cm_loop:     jsr read_byte
             sta load,y
             iny
             cpy #19
             bne cm_loop       ;read header

             ldx #7
             lda #0
             
lb_loop:     ora banks,x       ;check to see if all bank bytes are 00h
             pha
             txa
             sta 5ff8h,x       ;and set banks up to 0,1,2,3,4,5,6,7
             pla
             dex
             bpl lb_loop
             cmp #0
             beq got_bank
             
             ldx #7

lb_loop2:    lda banks,x
             sta 5ff8h,x
             dex
             bpl lb_loop2
             lda #0
             sta 5ff8h
             sta temp2
             jmp fin_ld        ;if the banks were non-zero, load them up

got_bank:    lda load+1
             lsr a
             lsr a
             lsr a
             lsr a
             and #07h
             sta 5ff8h
             sta temp2         ;start loading at proper bank if non-banked
             
fin_ld:      lda load+1
             and #00fh
             sta load+1        ;strip to get location in bank

             lda load
             sta temp1_lo
             lda load+1
             and #0fh
             ora #080h
             sta temp1_hi      ;adjust to get offset into bank
             ldy #0
             
ld_lp1:      jsr read_byte
             sta (temp1),y
             inc temp1_lo
             bne do_next       ;load 256 bytes
             inc temp1_hi
             lda #090h
             cmp temp1_hi      ;load 4K banks
             bne do_next
             inc temp2
             lda temp2
             sta 5ff8h
             lda #080h
             sta temp1_hi      ;inc bank and reset pointers

do_next:     dec len
             lda #0ffh
             cmp len
             bne ld_lp1
             dec len+1
             cmp len+1
             bne ld_lp1
             dec len+2
             cmp len+2
             bne ld_lp1        ;dec length counter
             
             lda banks
             sta 5ff8h         ;fix first bank we messed with

runit:       ldx #0h
             
ld_def:      lda init_sound,x
             sta 04000h,x      ;init sound regs
             inx
             cpx #14h
             bne ld_def

             lda #00fh
             sta 04015h        ;turn all chans on

             ldx #0fh
             
ld_nop:      lda nsf_play,x
             sta inbase,x
             dex
             bpl ld_nop        ;load code into 01f0h
             
             ldx #0efh
             txs               ;new stack
             
             lda init
             sta inbase+1
             lda init+1
             sta inbase+2
             lda play
             sta inbase+9
             lda play+1
             sta inbase+10     ;fix up our code
             

             ldy initsng
             
             ldx #44

fc_ld:       lda final_clr,x
             sta 0100h,x
             dex
             bpl fc_ld
             jmp 0100h


final_clr:   ldx #0
             txa

gb_clr:      sta 0,x 
             sta 00efh,x    ;don't overwrite code
             sta 0200h,x
             sta 0300h,x
             sta 0400h,x
             sta 0500h,x
             sta 0600h,x
             sta 0700h,x
             dex
             bne gb_clr     ;clear zeropage
             lda #0fch
             sta 04800h     ;turn off copier crap
             tya
             clc
             sbc #0
             tax
             tay
             jmp inbase


init_sound:  .db 0,0,0,0
             .db 0,0,0,0
             .db 0,0,0,0
             .db 010h,0,0,0
             .db 0,0,0,0
             
;gets loaded into 01F0h

nsf_play:    jsr 0000h      ;0
             
ns_loop1:    lda 2002h      ;3
             bpl ns_loop1   ;6
             jsr 0000h      ;8
             beq ns_loop1   ;b
             bne ns_loop1   ;d
             nop            ;f

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data



             .end
